from spider import Spider
import re
from evaluator import Evaluator
from extractor import Extractor

class BilibiliSpider(Spider):
    def __init__(self, database_path, sleep_time=15, mode="external", debug=True, load_old=False):
        super().__init__(database_path, sleep_time, mode, debug, load_old)
        
        # 构造初始url
        # https://www.bilibili.com/read/cv22990000 从2023-04-11 16:27开始往前爬
        # https://www.bilibili.com/read/cv20000000 爬到2022-11-24 13:57 共5个月

        # 太慢了
        # for i in range(22990000,20000000,-1):
        #     print(i)
        #     if i%100000==0:
        #         print(i)
        #     self.add_url('https://www.bilibili.com/read/cv'+str(i))

        if 'www.bilibili.com' not in self.urls:
            self.urls['www.bilibili.com']=[]

        # 重置
        if len(self.urls['www.bilibili.com'])<10:
            self.urls['www.bilibili.com']=['https://www.bilibili.com/read/cv'+str(i) for i in range(20000000,22990001)]
            self.visited_urls={}
        

    # 只对满足other_rule的url的HTML进行提取文本(pick_webtext)
    def other_rule(self,url):
        return 'read' in url and "cv" in url
    
    # 被ban的url将不会访问
    def ban(self,url):
        return 'read' not in url
    
    def pick_webtext(self,html):
        threshold = 5
        if '页面不存在或已被删除' in html:
            return []
        
        if not html:
            return []

        # 获取点赞数
        upvote_re = r'like":[\s\S]*?"dislike"'
        upvote = re.findall(upvote_re,html)[0]
        upvote = re.findall(r'\d+',upvote)[0]           
        upvote = int(upvote)   
        # print(upvote)
        # print(html)

        if upvote < threshold:
            return []

        # 获取正文
        content_re = r'"content":"[\s\S]*?","keywords":"'
        content = re.findall(content_re,html)[0]

        content = re.sub(r'"content":"','',content)
        content = re.sub(r'","keywords":"','',content)


        # content = re.sub(r'{\\"ops\\":\[{\\"insert\\":\\"','',content)
        # content = re.sub(r'\\"}]}','',content)
        # content = content.strip(r'"content":"')
        # content = content.strip(r'","keywords":"')
        # content = content.strip(r'{\\"ops\\":[{\\"insert\\":\\"')
        # content = content.strip(r'\\"}]}')



        if r'\"insert\":\"' in content:
            reg = r'{\\"insert\\":\\"[\s\S]*?\\"}'
            content = re.findall(reg,content)
            str_list = []
            for c in content:
                # print(c)
                # print(c[14:-3])
                str_list.append(c[14:-3])
            content = '\n'.join(str_list)

        # print([content])
        

        # 删标签
        content = re.sub(r'</h\d>|</p>','\n',content)
        
        content = re.sub(r'\\u003C\\u002Fp\\u003E','\n',content)
        content = re.sub(r'\\u003c\\u002fp\\u003e','\n',content)
        content = re.sub(r'\\u003C\\u002Fh\d\\u003E','\n',content)
        content = re.sub(r'\\u003c\\u002fh\d\\u003e','\n',content)

        content = re.sub(r'\\u003Ch\d\\u003E','\n',content)
        content = re.sub(r'\\u003ch\d\\u003e','\n',content)
        content = re.sub(r'\\u003Cp\\u003E','\n',content)
        content = re.sub(r'\\u003cp\\u003e','\n',content)

        content = re.sub(r'<[^>]+>','',content)
        content = re.sub(r'\\u003C[\s\S]*?\\u003E','',content)
        content = re.sub(r'\\u003c[\s\S]*?\\u003e','',content)
        # print([content])
        # 把unicode转回str
        content = re.sub(r'\\u002F',r'/',content)
        content = re.sub(r'\\u002f',r'/',content)
        content = re.sub(r'\\u002C',r'<',content)
        content = re.sub(r'\\u002c',r'<',content)
        content = re.sub(r'\\u002E',r'>',content)
        content = re.sub(r'\\u002e',r'>',content)
        content = re.sub(r'\\\\n','\n',content)
        content = re.sub(r'\\\\t','\t',content)
        content = re.sub(r'\\\\\\"',r'"',content)
        content = re.sub(r'\\\\',r'\\',content)
        content = re.sub(r'\\n',r'\n',content)
        content = re.sub(r'\\t',r'\t',content)
        
        
        # html实体转回字符
        import html
        content = html.unescape(content)
        del html 

        # 删空行
        content = re.sub(r'\s*\n\s*','\n',content) 
        return [content.lstrip('\n')]
    
class CSDNSpider(Spider):
    def __init__(self, database_path, sleep_time=15, mode="external", debug=True, load_old=False):
        super().__init__(database_path, sleep_time, mode, debug, load_old)

    def other_rule(self,url):
        return 'article/details' in url
    def pick_webtext(self,html):
        threshold = 5
        
        if not html:
            return []
        # 获取点赞数
        upvote_re = r'<span id="spanCount" class="count ">[\s\S]*?</span>'
        upvote = re.findall(upvote_re,html)[0]
        upvote = re.findall(r'\d+',upvote)[0]           
        upvote = int(upvote)   
        # print(upvote)
        # print(html)

        if upvote < threshold:
            return []

        # 获取正文
        content_re = r'<article class="baidu_pl">[\s\S]*?</article>'
        content = re.findall(content_re,html)[0]

        # 分割代码
        content = re.split("(<code class=[\s\S]*?</code>)",content)

        return_texts = []
        for c in content:

            # 处理代码内容
            if "<code class=" in c:
                
                # 获取语言
                # r"""<code class="prism language-python">"""
                # print(c)
                language = c.split("""language-""")
                # print("debug:",len(language))
                if len(language) == 1:
                    language = "code"
                else:
                    language = language[1]
                language = language.split('\"')[0]
                # print(language)
                
                # 删标签
                c = re.sub(r'<[^>]+>','',c)

                # 将HTML实体转回字符
                import html
                c = html.unescape(c)
                del html

                # print("<"+language+">\n"+c+r"</"+language+">")
                # print('---'*40)

                return_texts.append("<"+language+">\n"+c+"\n</"+language+">")
            # 处理文本内容
            else:

                # 获取latex
                latex_re = r'(<span class="katex-mathml">[\s\S]*?</span>)'
                c_list = re.split(latex_re,c)

                
                for ind in range(len(c_list)):
                    # 提取latex
                    if r'<span class="katex-mathml">' in c_list[ind]:
                        c_list[ind] = re.sub(r'<span class="katex-mathml">','\n',c_list[ind])
                        # c_list[ind] = re.sub('\n\s','\n',c_list[ind])
                        # c_list[ind] = re.findall(r'[^\n]{30,}',c_list[ind])
                        c_list[ind] = c_list[ind].split('\n')

                        # latex表达式最长
                        c_list[ind] = sorted(c_list[ind],key=lambda x:-len(x))[0].strip()

                        # print([c_list[ind]])
                        # print('-'*40)
                    # 删除latex渲染
                    elif r'<span class="katex-html">' in c_list[ind]:
                        # print(c_list[ind])
                        c_list[ind] = re.sub(r'</h\d>|</p>|</br>|</title>|</li>|<br />','\n',c_list[ind])
                        c_list[ind] = c_list[ind].split('\n')[1:]
                        c_list[ind] = '\n'.join(c_list[ind])
                        # print(c_list[ind])
                        # print('-'*40)
                        
                     
                c = '\n'.join(c_list)
                # print(c) 
                # print('^^^'*40)

                # 删标签
                c = re.sub(r'</h\d>|</p>|</br>|</title>|</li>|<br />','\n',c)
                # print(c) 
                # print('==='*40)
                c = re.sub(r'<[^>]+>','',c)  

                # print(c) 
                # print('???'*40)               


                # 删空行
                c = re.sub(r'\s*\n\s*','\n',c)  

                # 将HTML实体转回字符
                import html
                c = html.unescape(c)
                del html    
                
                # print(c) 
                # print('---'*40)
                return_texts.append(c)

        # 按顺序拼回全文
        return_texts = '\n'.join(return_texts)
        
        # 删空行
        return_texts = [return_texts.strip('\n')]
        return return_texts
    

class ZhihuSpider(Spider):
    def __init__(self, database_path, sleep_time=15, mode="external", debug=True):
        super().__init__(database_path, sleep_time, mode, debug)

    def other_rule(self,url):
        # 对要爬取的url做出约束,在该约束下只爬取带有'question'的url的文本;
        # 注意:不带有'question'的url链接仍然会爬取,他们的作用是爬取更多的链接。
        return 'question' in url 
    def pick_webtext(self,html):
        threshold = 50
        important_texts = []
        return_texts = []
        re2 = r'(赞同 \d+)'
        blocks = re.split(re2, html)
        for i,block in enumerate(blocks):
            if block.startswith('赞同'):
                if not important_texts[-1].isdigit():
                    important_texts.append(re.findall(r'\d+',block)[0])
                
                
            elif i+1<len(blocks) and blocks[i+1].startswith('赞同'):
                
                
                
                re1 = r'[\s\S]+[。？！]'
                p = re.findall(re1,block)
                
                
                # 如果有文本中一些英文被删了，尝试调大后面的参数
                reg=r'[\s\da-zA-Z’!"\"#\$%&\(\)\*\+,\-./:;<=>\?@\[\]\^_`\{\|\}~。！，\\\n\t\']{40,}'
                
    #             reg=r'<[^>]+>'
                
                if p:
                    for s in p[-1:]:
                        
                        s = re.sub(reg,'\n',s)
                        
                        s = re.sub(r'</h\d>|</p>','\n',s)
                        
                        s = re.sub(r'<[^>]+>','',s) 
    #                     print(s)
                        important_texts.append(s.split('赞同了该回答')[-1].strip('\n•'))        
        
        for i in range(0,len(important_texts),2):
            if int(important_texts[i+1]) > threshold:
                return_texts.append(important_texts[i])
                
        return return_texts


class RobustSpider(Spider):
    def __init__(self, database_path, sleep_time=15, evaluator=Evaluator(), extractor=Extractor(),mode="external", debug=True, load_old=False):
        super().__init__(database_path, sleep_time, mode, debug, load_old)
        self.evaluator = evaluator
        self.extractor = extractor
    
    def pick_webtext(self,html):
        content = self.extractor.extract_text(html=html)
        content = '\n'.join(content)

        # 删标签
        content = re.sub(r'</h\d>|</p>','\n',content)
        
        content = re.sub(r'\\u003C\\u002Fp\\u003E','\n',content)
        content = re.sub(r'\\u003c\\u002fp\\u003e','\n',content)
        content = re.sub(r'\\u003C\\u002Fh\d\\u003E','\n',content)
        content = re.sub(r'\\u003c\\u002fh\d\\u003e','\n',content)

        content = re.sub(r'\\u003Ch\d\\u003E','\n',content)
        content = re.sub(r'\\u003ch\d\\u003e','\n',content)
        content = re.sub(r'\\u003Cp\\u003E','\n',content)
        content = re.sub(r'\\u003cp\\u003e','\n',content)

        content = re.sub(r'<[^>]+>','',content)
        content = re.sub(r'\\u003C[\s\S]*?\\u003E','',content)
        content = re.sub(r'\\u003c[\s\S]*?\\u003e','',content)
        # print([content])
        # 把unicode转回str
        content = re.sub(r'\\u002F',r'/',content)
        content = re.sub(r'\\u002f',r'/',content)
        content = re.sub(r'\\u003C',r'<',content)
        content = re.sub(r'\\u003c',r'<',content)
        content = re.sub(r'\\u003E',r'>',content)
        content = re.sub(r'\\u003e',r'>',content)
        content = re.sub(r'\\\\n','\n',content)
        content = re.sub(r'\\\\t','\t',content)
        content = re.sub(r'\\\\\\"',r'"',content)
        content = re.sub(r'\\\\',r'\\',content)
        content = re.sub(r'\\n',r'\n',content)
        content = re.sub(r'\\t',r'\t',content)
        
        
        # html实体转回字符
        import html
        content = html.unescape(content)
        del html 

        # 删空行
        content = re.sub(r'\s*\n\s*','\n',content) 

        content = content.lstrip('\n').split('\n')
        return_texts = []
        # 拼回连续文本
        text = ""
        for line in content:
            if not line:
                continue
            if not text:
                text = line 
            else:
                # print("continues:",evaluator.predict(text1=text,text2=line))
                if self.evaluator.is_continuous(text,line):
                    text += line
                # 高赞的概率>0.7
                elif self.evaluator.is_good(text):
                    # 只加入长文本
                    if len(text)>10:
                        return_texts.append(text)
                    text = line
                else:
                    text = line
        if text and self.evaluator.is_good(text):
            return_texts.append(text)
        return return_texts


import requests
import json
import time
import os
import time
import json
import urllib
from evaluator import Evaluator
from extractor import Extractor



class Spider:

    

    def __init__(self, database_path, sleep_time=15, mode="external", debug=True, load_old=False):
        self.database_path = database_path
        #folder = os.path.exists(self.database_path)
        self.sleep_time = sleep_time
        self.mode= mode
        self.debug = debug
        self.urls = {}
        self.visited_urls = {}
        self.docs = {}  #Dict[str,Dict[str,list[str]]] i.e. {域名:{url:文本列表}}
        self.last_get_html_time=0
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36 Edg/111.0.1661.62'
        }
        #创建原始路径
        if not os.path.exists(self.database_path):     
            os.makedirs(self.database_path)

        #导入urls库数据
        # url_database_database_path=self.database_path+'url_database/database/'
        # if  os.path.exists(url_database_database_path):     
        #     filelist = os.listdir(url_database_database_path)
        #     for filename in filelist:
        #         # print(filename)
        #         filename=filename[:-4]
        #         if filename not in self.urls:
        #             self.urls[filename] = []
        #         with open (url_database_database_path+filename +'.txt','r',encoding='utf-8') as file:
        #             content=file.readlines()
        #             for line in content:
        #                 if line not in self.urls[filename]:
        #                     self.urls[filename].append(line.strip('\n'))

        if load_old:
            #导入visited_urls
            if os.path.exists(os.path.join(database_path,'visited_urls.json.old')):
                with open(os.path.join(database_path,'visited_urls.json.old')) as f_obj:
                    self.visited_urls = json.load(f_obj)
            #导入urls
            if os.path.exists(os.path.join(database_path,'urls.json.old')):
                with open(os.path.join(database_path,'urls.json.old')) as f_obj:
                    self.urls = json.load(f_obj)
            #导入visited_urls
            if os.path.exists(os.path.join(database_path,'docs.json.old')):
                with open(os.path.join(database_path,'docs.json.old')) as f_obj:
                    self.docs = json.load(f_obj)           
        else:
            #导入visited_urls
            if os.path.exists(os.path.join(database_path,'visited_urls.json')):
                with open(os.path.join(database_path,'visited_urls.json')) as f_obj:
                    self.visited_urls = json.load(f_obj)
            #导入urls
            if os.path.exists(os.path.join(database_path,'urls.json')):
                with open(os.path.join(database_path,'urls.json')) as f_obj:
                    self.urls = json.load(f_obj)
            #导入visited_urls
            if os.path.exists(os.path.join(database_path,'docs.json')):
                with open(os.path.join(database_path,'docs.json')) as f_obj:
                    self.docs = json.load(f_obj)
       
       

        # #导入doc数据库
        # text_database_database_path=self.database_path+'text_database/database/'
        # if  os.path.exists(text_database_database_path):     
        #     filelist = os.listdir(text_database_database_path)
        #     for filename in filelist:
        #         with open (text_database_database_path+filename,'r',encoding='utf-8') as file:
        #             lines=file.readlines()
        #             for line in lines:
        #                 content=line.split('<eod><bod>')
        #                 url=content[0].split('<bod>')[0].split('<bop>')[1]
        #                 content[0]=content[0].split('<bod>')[1]
        #                 content[-1]=content[-1].split('<eod>')[0]
        #                 self.docs[url] = content

    def other_rule(self,url):
        return True
    
    def ban(self, url):
        return False

    def is_valid(self, url):
        is_pure_url = '#' not in url and '?' not in url and '=' not in url and '%' not in url and '+' not in url and '&' not in url and ' ' not in url 
        is_normal_domain = '.com' in url or '.net' in url
        is_link = 'js' not in url and 'css' not in url and 'png' not in url and 'jpg' not in url and 'htm'not in url
        is_content = 'gov' not in url and 'edu' not in url 
        return is_pure_url and is_normal_domain and is_link and is_content 
    
    def add_url(self, url, force=True):
        """
            默认外部添加的url是合法的
        """

        if self.ban(url):
            return

        # print("add:",url)
        domain = self.get_domain(url)
        if force:
            if domain not in self.urls:
                self.urls[domain] = []

            if domain not in self.visited_urls:
                self.visited_urls[domain] = []
                
            if url not in self.urls[domain]:
                if 'https' in url:
                    if 'http'+url[5:] not in self.urls[domain] and 'http'+url[5:] not in self.visited_urls[domain]:
                        self.urls[domain].append(url) 
                elif 'http' in url:
                    if 'https'+url[4:] not in self.urls[domain] and 'https'+url[4:] not in self.visited_urls[domain]:
                        self.urls[domain].append(url) 

        elif self.is_valid(url):
            if self.mode == "external" or (self.mode == "inner" and domain in self.urls):
                if domain not in self.urls:
                    self.urls[domain] = []
                if domain not in self.visited_urls:
                    self.visited_urls[domain] = []
                if url not in self.urls[domain]:
                    if 'https' in url:
                        if 'http'+url[5:] not in self.urls[domain] and 'http'+url[5:] not in self.visited_urls[domain]:
                            self.urls[domain].append(url) 
                    elif 'http' in url:
                        if 'https'+url[4:] not in self.urls[domain] and 'https'+url[4:] not in self.visited_urls[domain]:
                            self.urls[domain].append(url)             
         
        

    def get_urls(self):
        return_urls = []
        for domain, links in self.urls.items():
            if domain not in self.visited_urls:
                self.visited_urls[domain]=[]
            # print("wai:",links)
            # print("bool:",bool(links))
            while links:
                link = links.pop()
                # print("内:",links)
                # print(self.urls[domain])
                if link not in self.visited_urls[domain]: 
                    self.visited_urls[domain].append(link)
                    return_urls.append(link)
                    break
        return return_urls

    def pick_webtext(self, html):
        """
        return List[str] : cherry-pick webtext you want get in html
        """
        extractor=Extractor()
        return extractor.extract_text(html)


    def crawl(self):
        extractor=Extractor()
        while 1:
            geted_urls=self.get_urls()
            if len(geted_urls) == 0:
                self.save()
                break
            
            for url in geted_urls:
                # print('爬取：',url)
                html=self.get_html(url)
                # print(html)
                need_add_urls=extractor.extract_url(html)
                # print("need:",need_add_urls)
                for need_add_url in need_add_urls:
                    self.add_url(need_add_url,force=False)
                
                if self.other_rule(url):
                    cleaned_texts = self.pick_webtext(html) #list[str]
                    if not cleaned_texts:
                        continue
                    domain = self.get_domain(url)
                    if domain not in self.docs:
                        self.docs[domain]={}
                    self.docs[domain][url]=cleaned_texts
                    
            self.save()
            time.sleep(self.sleep_time)


        

    def get_html(self, url: str) :
        


        try:
            r = requests.get(url, headers=self.headers, timeout=15)
        except:
            return ""
        if r.status_code == 200:
            r.encoding = r.apparent_encoding
            return r.text
        else:
            return ""

    # def deduplicate(self,test_doc:str ) :#判断相似性
    #     evaluator = Evaluator()
    #     for domain,doc in self.docs.items():
    #         if evaluator.predict(test_doc,doc)>0.7:
    #             return 1
    #     return 0

                    
    def save(self) :
        # #url数据库路径创建
        # url_database_path = self.database_path+'url_database/'
        # url_website_path =url_database_path+'website.txt'
        # url_database_database_path=url_database_path+'database/'
        # #folder1 = os.path.exists(url_database_path)
        # if not os.path.exists(url_database_path):     
        #     os.makedirs(url_database_path)
        # #folder2 = os.path.exists(url_database_database_path)
        # if not os.path.exists(url_database_database_path):     
        #     os.makedirs(url_database_database_path)

        # #text数据库路径创建
        # text_database_path = self.database_path+'text_database/'
        # text_website_path =text_database_path+'website.txt'
        # text_database_database_path=text_database_path+'database/'
        
        # if not os.path.exists(text_database_path):     
        #     os.makedirs(text_database_path)
        
        # if not os.path.exists(text_database_database_path):     
        #     os.makedirs(text_database_database_path)

        # #写入url数据库
        # print('写入url数据库')
        # with open(url_website_path,"w") as f:
        #     for domain in self.urls:
        #         f.write(domain+'\n')

        # for domain in self.urls:
        #     with open(url_database_database_path+'/'+domain+'.txt',"w") as f:
        #         for url in self.urls [domain]:               
        #             f.write(url+'\n')

        # #写入doc数据
        # print('写入doc数据库')
        # with open(text_website_path,"w") as f:
        #     for url in self.docs:
        #         domain=self.get_domain(url)
        #         f.write(domain+'\n')

        # for url in self.docs:
        #     domain=self.get_domain(url)
        #     with open(text_database_database_path+'/'+domain+'.txt',"w") as f:
        #         f.write('<bop>'+url)
        #         for doc in self.docs[url]:   
        #             print('**')
        #             print(doc)   
        #             print(type(doc))         
        #             f.write('<bod>'+doc+'<eod>')
        #         f.write('<eop>\n')

        #保存visited_urls
        # print('保存json')

        file_name = os.path.join(self.database_path,'visited_urls.json')
        if os.path.exists(file_name):
            if os.path.exists(file_name+'.old'):
                os.unlink(file_name+'.old')         
            os.rename(file_name,file_name+'.old')

        file_name = os.path.join(self.database_path,'urls.json')
        if os.path.exists(file_name):
            if os.path.exists(file_name+'.old'):
                os.unlink(file_name+'.old')
            os.rename(file_name,file_name+'.old')

        file_name = os.path.join(self.database_path,'docs.json')
        if os.path.exists(file_name):
            if os.path.exists(file_name+'.old'):
                os.unlink(file_name+'.old')
            os.rename(file_name,file_name+'.old') 

        with open(os.path.join(self.database_path,'visited_urls.json'), 'w') as f_obj:
            json.dump(self.visited_urls, f_obj, ensure_ascii=False)
        
        with open(os.path.join(self.database_path,'urls.json'), 'w') as f_obj:
            json.dump(self.urls, f_obj, ensure_ascii=False)
        
        with open(os.path.join(self.database_path,'docs.json'), 'w') as f_obj:        
            json.dump(self.docs, f_obj, ensure_ascii=False)
         
                

    def get_domain(self, url: str) :
        return urllib.parse.urlparse(url).netloc



import torch
from transformers import BertTokenizer, AlbertModel
from transformers import AutoModelForSequenceClassification



class Evaluator:
    def __init__(self, tokenizer=None, model=None, threshold=0.7, cuda="cpu"):
        self.device = torch.device(cuda if torch.cuda.is_available() else "cpu")
        self.tokenizer =tokenizer
        self.model=model
        self.threshold = threshold
        if not self.tokenizer:
            self.tokenizer = BertTokenizer.from_pretrained("uer/albert-large-chinese-cluecorpussmall")
        if not self.model:
            self.model = AutoModelForSequenceClassification.from_pretrained("uer/albert-large-chinese-cluecorpussmall", num_labels=2)
            self.model.load_state_dict(torch.load("large.pt",map_location=self.device))
            self.model.to(self.device)

    def predict(self,text1,text2=None):
        self.model.eval()
        
        encoded_input = self.tokenizer(text1[-32:],text2[:32], padding=True ,return_tensors='pt').to(self.device) \
    if text2 else self.tokenizer(text1[:64], padding=True ,return_tensors='pt').to(self.device)
        
        output = self.model(**encoded_input)
        prob = torch.softmax(output.logits,-1)
        p = prob[0][1].item()
        return p
    
    
    def is_good(self,text):
        return self.predict(text)>self.threshold
    
    def is_continuous(self,text1, text2):
        return self.predict(text1,text2)>self.threshold
    
    # def __call__(self,text1,text2):
    #     return self.predict(text1,text2)>self.threshold
    


import re 
class Extractor:
    def extract_url(self, html: str) :
        pattern = re.compile(r'<a\s+(?:[^>]*?\s+)?href="([^"]*)"')
        urls = pattern.findall(html)
        #print('urls',urls)
        valid_urls = []
        for url in urls:
            if url.startswith('http') or url.startswith('https') or url.startswith('www.') or  url.startswith('//www.'):
                if(url[0:2] == '//'):
                    url='http:'+url
                valid_urls.append(url)
        return valid_urls

    def extract_text(self, html: str):
        #reg=r'[\s\da-zA-Z’!"\"#\$%&\(\)\*\+,\-./:;<=>\?@\]\^_`\{\|\}~。！，\\\n\t\']{10,}'
        reg=r'[\s\da-zA-Z’!\"#\$%&\*\+,\-./:;<=>\?@\]\^_`\{\|\}~。！，`~!@\^\(\)-_=+\|\\\n\t\']{10,}'
        #reg=r'
        # ;:\':,.<>/?'
        p=re.sub(reg,'\n',html)

        # 按换行,制表符给文章分段
        p=re.split(r'\t\n',p)
        valid_docs=[]
        for s in p:
            # 找出所有以[。？！]结尾的字符串
            #print("*"*80)
    
            s = re.findall(r'[\s\d\w《》？：“”+——）（*&……%￥#@！~【】；‘，。’’!"\"#\$%&\(\)\*\+,\-./:;<=>\?@\]\^_`\{\|\}~。！，\\\n\t\']+[。？！]',s)
            
            for c in s:
                valid_docs.append(c)
                   

        return valid_docs




#!/usr/bin/env python
from io import open
from setuptools import setup, find_packages
setup(
    name='Chinese-webtext-spider',
    version='1.0.0',
    description='a package for crawling chinese webtext',
    long_description='一个用于爬取网络中文文本的爬虫工具',
    author='ZhaoRunSong',
    author_email='1398915234@qq.com',
    license='Apache License 2.0',
    url='https://github.com/1azybug/chinese-webtext-spider',
    download_url='https://github.com/1azybug/chinese-webtext-spider/master.zip',
    packages=find_packages(),
    install_requires=['requests','torch','transformers']
)